/*
 * Example code for using /dev/modplay
 *
 * (c) 1999 draco@atari.org
 */

#include <stdio.h>
#include <mintbind.h>
#include <file.h>

/* Opcodes for Fcntl()
 */

/* arg == 0, start playing
 * arg == -1, interrogate the player state (0 idle, 1 playing)
 * arg > 0 reserved, don't use
 */
# define MODULE_PLAY	('A'<<8|0)

/* Stop playing, arg == 0
 */
# define MODULE_STOP	('A'<<8|1)

/* Flush the module, release memory, arg == 0
 */
# define MODULE_FLUSH	('A'<<8|2)

/* Toggle FX, `arg' is bitmap with bits defined as below
 */
# define MODULE_FX	('A'<<8|3)
#  define SURROUND	1
#  define INTERPOLATION	2

/* Set the surround delay to `arg' (default is $0800)
 */
# define MODULE_SRDELAY	('A'<<8|4)

/* Revert to defaults, arg == 0
 * DOES NOT FLUSH THE MODULE, only resets FX, surround delay etc.
 */
# define MODULE_RESET	('A'<<8|5)

# define DEV	"u:\\dev\\modplay"

int
main(int argc, char *argv[])
{
	short fd;
	char *buf;
	long r;
	XATTR atr;

	if (argc == 1) {
		printf("Flushing the modplay buffer... ");
		fd = Fopen(DEV, O_WRONLY);
		(void)Fcntl(fd, 0L, MODULE_FLUSH);
		(void)Fcntl(fd, 0L, MODULE_RESET);
		Fclose(fd);
		printf("OK\n");

		return 0;
	}

	r = Fxattr(0, argv[1], &atr);
 
	if (r < 0) {
		printf("%s: error %ld\n", argv[1], r);
		return r;
	}

	if (!atr.size) {
		printf("%s: null file (?)\n", argv[1]);
		return 0;
	}

	buf = (char *)Mxalloc(atr.size, 3);
	fd = Fopen(argv[1], 0);

	if (fd < 0) {
		printf("%s: error %d\n", argv[1], fd);
		return fd;
	}

	r = Fread(fd, atr.size, buf);
	Fclose(fd);

	if (r < 0) {
		printf("%s: error %ld\n", argv[1], r);
		return r;
	}
	
	printf("Module loaded OK\nOpening modplayer...");

	fd = Fopen(DEV, O_WRONLY);

	if (fd < 0) {
		printf("failed, error %d\n", fd);
		return fd;
	}

	r = Foutstat(fd);

	if (r < 0) {
		printf("not ready to receive data, error %ld\n", r);
		return r;
	}

	if (r == 0 || (atr.size > r)) {
		printf("out of memory (?)\n");
		return 0;
	}

	printf("OK, %ld bytes available\n", r);

	printf("Status: ");

	printf("%s\n", (Fcntl(fd, -1L, MODULE_PLAY) == 0) ? "idle" : "playing");

	printf("Uploading module... ");
	fflush(stdout);

	r = Fwrite(fd, atr.size, buf);		/* must be atomic write */

	Mfree(buf);				/* buffer not neccessary anymore */

	if (r == 0) {
		printf("data refused (?)\n");
		return 0;
	}

	if (r < 0) {
		printf("error %ld\n", r);
		return 0;
	}

	printf("%ld bytes OK\n", r);

	r = Fcntl(fd, INTERPOLATION|SURROUND, MODULE_FX);
	printf("Surround/interpolation on, status %ld\n", r); 

	r = Fcntl(fd, 0x0800, MODULE_SRDELAY);
	printf("Setting surround delay, status %ld\n", r);

	r = Fcntl(fd, 0L, MODULE_PLAY);
	printf("Start playing, status %ld\n", r);

	if (r < 0) {
		Fcntl(fd, 0L, MODULE_FLUSH);
		Fcntl(fd, 0L, MODULE_RESET);
	}

	Fclose(fd);

	printf("Goodbye\n");

	return 0;
}
